// ====================================================================
// This file is part of FlexibleSUSY.
//
// FlexibleSUSY is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published
// by the Free Software Foundation, either version 3 of the License,
// or (at your option) any later version.
//
// FlexibleSUSY is distributed in the hope that it will be useful, but
// WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with FlexibleSUSY.  If not, see
// <http://www.gnu.org/licenses/>.
// ====================================================================

// File generated at Sat 22 Nov 2014 11:02:29

/**
 * @file MRSSM_two_scale_model.hpp
 * @brief contains class for model with routines needed to solve boundary
 *        value problem using the two_scale solver by solvingt EWSB
 *        and determine the pole masses and mixings
 *
 * This file was generated at Sat 22 Nov 2014 11:02:29 with FlexibleSUSY
 * 1.0.3 and SARAH 4.4.0 .
 */

#ifndef MRSSM_TWO_SCALE_H
#define MRSSM_TWO_SCALE_H

#include "MRSSM_model.hpp"
#include "MRSSM_two_scale_soft_parameters.hpp"
#include "MRSSM_physical.hpp"
#include "MRSSM_info.hpp"
#include "two_scale_model.hpp"
#include "higgs_2loop_corrections.hpp"
#include "problems.hpp"
#include "config.h"

#include <iosfwd>
#include <string>

#ifdef ENABLE_THREADS
#include <mutex>
#endif

#include <gsl/gsl_vector.h>
#include <gsl/gsl_multiroots.h>
#include <Eigen/Core>

namespace flexiblesusy {

class Two_scale;
/**
 * @class MRSSM<Two_scale>
 * @brief model class with routines for determing masses and mixinga and EWSB
 */
template<>
class MRSSM<Two_scale> : public Two_scale_model, public MRSSM_soft_parameters {
public:
   explicit MRSSM(const MRSSM_input_parameters& input_ = MRSSM_input_parameters());
   virtual ~MRSSM();

   void calculate_DRbar_masses();
   void calculate_DRbar_parameters();
   void calculate_pole_masses();
   virtual void clear();
   void clear_DRbar_parameters();
   void do_calculate_sm_pole_masses(bool);
   bool do_calculate_sm_pole_masses() const;
   void reorder_DRbar_masses();
   void reorder_pole_masses();
   void set_ewsb_iteration_precision(double);
   void set_ewsb_loop_order(unsigned);
   void set_higgs_2loop_corrections(const Higgs_2loop_corrections&);
   void set_number_of_ewsb_iterations(std::size_t);
   void set_number_of_mass_iterations(std::size_t);
   void set_pole_mass_loop_order(unsigned);
   void set_physical(const MRSSM_physical&);
   double get_ewsb_iteration_precision() const;
   double get_ewsb_loop_order() const;
   const MRSSM_physical& get_physical() const;
   MRSSM_physical& get_physical();
   const Problems<MRSSM_info::NUMBER_OF_PARTICLES>& get_problems() const;
   Problems<MRSSM_info::NUMBER_OF_PARTICLES>& get_problems();
   int solve_ewsb_tree_level();
   int solve_ewsb_one_loop();
   int solve_ewsb();            ///< solve EWSB at ewsb_loop_order level

   // interface functions
   virtual void calculate_spectrum();
   virtual std::string name() const;
   virtual void run_to(double scale, double eps = -1.0);
   virtual void print(std::ostream&) const;
   virtual void set_precision(double);

   double get_lsp(MRSSM_info::Particles&) const;

   double get_MGlu() const { return MGlu; }
   const Eigen::Array<double,3,1>& get_MFv() const { return MFv; }
   double get_MSOc() const { return MSOc; }
   double get_MVZ() const { return MVZ; }
   const Eigen::Array<double,6,1>& get_MSd() const { return MSd; }
   const Eigen::Array<double,3,1>& get_MSv() const { return MSv; }
   const Eigen::Array<double,6,1>& get_MSu() const { return MSu; }
   const Eigen::Array<double,6,1>& get_MSe() const { return MSe; }
   const Eigen::Array<double,4,1>& get_Mhh() const { return Mhh; }
   const Eigen::Array<double,4,1>& get_MAh() const { return MAh; }
   const Eigen::Array<double,2,1>& get_MRh() const { return MRh; }
   const Eigen::Array<double,4,1>& get_MHpm() const { return MHpm; }
   const Eigen::Array<double,2,1>& get_MRpm() const { return MRpm; }
   const Eigen::Array<double,4,1>& get_MChi() const { return MChi; }
   const Eigen::Array<double,2,1>& get_MCha1() const { return MCha1; }
   const Eigen::Array<double,2,1>& get_MCha2() const { return MCha2; }
   const Eigen::Array<double,3,1>& get_MFe() const { return MFe; }
   const Eigen::Array<double,3,1>& get_MFd() const { return MFd; }
   const Eigen::Array<double,3,1>& get_MFu() const { return MFu; }
   double get_MVG() const { return MVG; }
   double get_MVP() const { return MVP; }
   double get_MVWm() const { return MVWm; }

   const Eigen::Matrix<double,6,6>& get_ZD() const { return ZD; }
   const Eigen::Matrix<double,3,3>& get_ZV() const { return ZV; }
   const Eigen::Matrix<double,6,6>& get_ZU() const { return ZU; }
   const Eigen::Matrix<double,6,6>& get_ZE() const { return ZE; }
   const Eigen::Matrix<double,4,4>& get_ZH() const { return ZH; }
   const Eigen::Matrix<double,4,4>& get_ZA() const { return ZA; }
   const Eigen::Matrix<double,2,2>& get_ZHR() const { return ZHR; }
   const Eigen::Matrix<double,4,4>& get_ZP() const { return ZP; }
   const Eigen::Matrix<double,2,2>& get_ZRP() const { return ZRP; }
   const Eigen::Matrix<std::complex<double>,4,4>& get_ZN1() const { return ZN1; }
   const Eigen::Matrix<std::complex<double>,4,4>& get_ZN2() const { return ZN2; }
   const Eigen::Matrix<std::complex<double>,2,2>& get_UM1() const { return UM1; }
   const Eigen::Matrix<std::complex<double>,2,2>& get_UP1() const { return UP1; }
   const Eigen::Matrix<std::complex<double>,2,2>& get_UM2() const { return UM2; }
   const Eigen::Matrix<std::complex<double>,2,2>& get_UP2() const { return UP2; }
   const Eigen::Matrix<std::complex<double>,3,3>& get_ZEL() const { return ZEL; }
   const Eigen::Matrix<std::complex<double>,3,3>& get_ZER() const { return ZER; }
   const Eigen::Matrix<std::complex<double>,3,3>& get_ZDL() const { return ZDL; }
   const Eigen::Matrix<std::complex<double>,3,3>& get_ZDR() const { return ZDR; }
   const Eigen::Matrix<std::complex<double>,3,3>& get_ZUL() const { return ZUL; }
   const Eigen::Matrix<std::complex<double>,3,3>& get_ZUR() const { return ZUR; }


   void calculate_MGlu();
   void calculate_MFv();
   void calculate_MSOc();
   void calculate_MVZ();
   Eigen::Matrix<double,6,6> get_mass_matrix_Sd() const;
   void calculate_MSd();
   Eigen::Matrix<double,3,3> get_mass_matrix_Sv() const;
   void calculate_MSv();
   Eigen::Matrix<double,6,6> get_mass_matrix_Su() const;
   void calculate_MSu();
   Eigen::Matrix<double,6,6> get_mass_matrix_Se() const;
   void calculate_MSe();
   Eigen::Matrix<double,4,4> get_mass_matrix_hh() const;
   void calculate_Mhh();
   Eigen::Matrix<double,4,4> get_mass_matrix_Ah() const;
   void calculate_MAh();
   Eigen::Matrix<double,2,2> get_mass_matrix_Rh() const;
   void calculate_MRh();
   Eigen::Matrix<double,4,4> get_mass_matrix_Hpm() const;
   void calculate_MHpm();
   Eigen::Matrix<double,2,2> get_mass_matrix_Rpm() const;
   void calculate_MRpm();
   Eigen::Matrix<double,4,4> get_mass_matrix_Chi() const;
   void calculate_MChi();
   Eigen::Matrix<double,2,2> get_mass_matrix_Cha1() const;
   void calculate_MCha1();
   Eigen::Matrix<double,2,2> get_mass_matrix_Cha2() const;
   void calculate_MCha2();
   Eigen::Matrix<double,3,3> get_mass_matrix_Fe() const;
   void calculate_MFe();
   Eigen::Matrix<double,3,3> get_mass_matrix_Fd() const;
   void calculate_MFd();
   Eigen::Matrix<double,3,3> get_mass_matrix_Fu() const;
   void calculate_MFu();
   void calculate_MVG();
   void calculate_MVP();
   void calculate_MVWm();

   double get_ewsb_eq_hh_1() const;
   double get_ewsb_eq_hh_2() const;
   double get_ewsb_eq_hh_3() const;
   double get_ewsb_eq_hh_4() const;

   std::complex<double> CpUSdconjUSdVZVZ(unsigned gO1, unsigned gO2) const;
   double CpUSdconjUSdconjSOcSOc(unsigned , unsigned ) const;
   double CpUSdconjUSdconjVWmVWm(unsigned gO1, unsigned gO2) const;
   std::complex<double> CpUSdconjUSdconjRhRh(unsigned gO1, unsigned gO2, unsigned gI1, unsigned gI2) const;
   std::complex<double> CpUSdconjUSdconjRpmRpm(unsigned gO1, unsigned gO2, unsigned gI1, unsigned gI2) const;
   double CpconjUSdbarCha1FuPR(unsigned , unsigned , unsigned ) const;
   std::complex<double> CpconjUSdbarCha1FuPL(unsigned gO1, unsigned gI1, unsigned gI2) const;
   std::complex<double> CpUSdconjUSdconjSvSv(unsigned gO1, unsigned gO2, unsigned gI1, unsigned gI2) const;
   std::complex<double> CpconjUSdFuCha2PR(unsigned gO2, unsigned gI1, unsigned gI2) const;
   std::complex<double> CpconjUSdFuCha2PL(unsigned gO1, unsigned gI1, unsigned gI2) const;
   std::complex<double> CpconjUSdFdChiPR(unsigned gO2, unsigned gI1, unsigned gI2) const;
   std::complex<double> CpconjUSdFdChiPL(unsigned gO1, unsigned gI1, unsigned gI2) const;
   std::complex<double> CpUSdconjUSdAhAh(unsigned gO1, unsigned gO2, unsigned gI1, unsigned gI2) const;
   std::complex<double> CpUSdconjUSdconjHpmHpm(unsigned gO1, unsigned gO2, unsigned gI1, unsigned gI2) const;
   std::complex<double> CpUSdconjUSdhhhh(unsigned gO1, unsigned gO2, unsigned gI1, unsigned gI2) const;
   std::complex<double> CpconjUSdbarChiFdPR(unsigned gO2, unsigned gI1, unsigned gI2) const;
   std::complex<double> CpconjUSdbarChiFdPL(unsigned gO1, unsigned gI1, unsigned gI2) const;
   std::complex<double> CpUSdconjUSdconjSdSd(unsigned gO1, unsigned gO2, unsigned gI1, unsigned gI2) const;
   std::complex<double> CpUSdconjUSdconjSeSe(unsigned gO1, unsigned gO2, unsigned gI1, unsigned gI2) const;
   std::complex<double> CpUSdconjUSdconjSuSu(unsigned gO1, unsigned gO2, unsigned gI1, unsigned gI2) const;
   std::complex<double> CpconjUSdSdRh(unsigned gO2, unsigned gI1, unsigned gI2) const;
   std::complex<double> CpconjUSdSuRpm(unsigned gO2, unsigned gI1, unsigned gI2) const;
   std::complex<double> CpconjUSdSdAh(unsigned gO2, unsigned gI1, unsigned gI2) const;
   std::complex<double> CpconjUSdSdhh(unsigned gO2, unsigned gI1, unsigned gI2) const;
   std::complex<double> CpconjUSdSuHpm(unsigned gO2, unsigned gI1, unsigned gI2) const;
   std::complex<double> CpconjUSdbarGluFdPR(unsigned gO2, unsigned , unsigned gI2) const;
   double CpconjUSdbarGluFdPL(unsigned , unsigned , unsigned ) const;
   double CpconjUSdGluFdPR(unsigned , unsigned , unsigned ) const;
   std::complex<double> CpconjUSdGluFdPL(unsigned gO1, unsigned , unsigned gI2) const;
   std::complex<double> CpconjUSdconjSOcSd(unsigned gO2, unsigned gI2) const;
   std::complex<double> CpconjUSdVGSd(unsigned gO2, unsigned gI2) const;
   std::complex<double> CpconjUSdVPSd(unsigned gO2, unsigned gI2) const;
   std::complex<double> CpconjUSdVZSd(unsigned gO2, unsigned gI2) const;
   std::complex<double> CpconjUSdVWmSu(unsigned gO2, unsigned gI2) const;
   std::complex<double> CpUSvconjUSvVZVZ(unsigned gO1, unsigned gO2) const;
   double CpUSvconjUSvconjVWmVWm(unsigned gO1, unsigned gO2) const;
   std::complex<double> CpUSvconjUSvconjRhRh(unsigned gO1, unsigned gO2, unsigned gI1, unsigned gI2) const;
   std::complex<double> CpUSvconjUSvconjRpmRpm(unsigned gO1, unsigned gO2, unsigned gI1, unsigned gI2) const;
   std::complex<double> CpconjUSvconjRpmSe(unsigned gO2, unsigned gI1, unsigned gI2) const;
   std::complex<double> CpUSvconjUSvconjSvSv(unsigned gO1, unsigned gO2, unsigned gI1, unsigned gI2) const;
   std::complex<double> CpconjUSvFeCha1PR(unsigned gO2, unsigned gI1, unsigned gI2) const;
   std::complex<double> CpconjUSvFeCha1PL(unsigned gO1, unsigned gI1, unsigned gI2) const;
   std::complex<double> CpconjUSvSvAh(unsigned gO2, unsigned gI1, unsigned gI2) const;
   std::complex<double> CpconjUSvSvhh(unsigned gO2, unsigned gI1, unsigned gI2) const;
   double CpconjUSvFvChiPR(unsigned , unsigned , unsigned ) const;
   std::complex<double> CpconjUSvFvChiPL(unsigned gO1, unsigned gI1, unsigned gI2) const;
   std::complex<double> CpUSvconjUSvAhAh(unsigned gO1, unsigned gO2, unsigned gI1, unsigned gI2) const;
   std::complex<double> CpUSvconjUSvconjHpmHpm(unsigned gO1, unsigned gO2, unsigned gI1, unsigned gI2) const;
   std::complex<double> CpUSvconjUSvhhhh(unsigned gO1, unsigned gO2, unsigned gI1, unsigned gI2) const;
   std::complex<double> CpconjUSvconjHpmSe(unsigned gO2, unsigned gI1, unsigned gI2) const;
   std::complex<double> CpUSvconjUSvconjSdSd(unsigned gO1, unsigned gO2, unsigned gI1, unsigned gI2) const;
   std::complex<double> CpUSvconjUSvconjSeSe(unsigned gO1, unsigned gO2, unsigned gI1, unsigned gI2) const;
   std::complex<double> CpUSvconjUSvconjSuSu(unsigned gO1, unsigned gO2, unsigned gI1, unsigned gI2) const;
   std::complex<double> CpconjUSvVZSv(unsigned gO2, unsigned gI2) const;
   std::complex<double> CpconjUSvconjVWmSe(unsigned gO2, unsigned gI2) const;
   std::complex<double> CpUSuconjUSuVZVZ(unsigned gO1, unsigned gO2) const;
   double CpUSuconjUSuconjSOcSOc(unsigned , unsigned ) const;
   double CpUSuconjUSuconjVWmVWm(unsigned gO1, unsigned gO2) const;
   std::complex<double> CpUSuconjUSuconjRhRh(unsigned gO1, unsigned gO2, unsigned gI1, unsigned gI2) const;
   std::complex<double> CpUSuconjUSuconjRpmRpm(unsigned gO1, unsigned gO2, unsigned gI1, unsigned gI2) const;
   double CpconjUSubarCha2FdPR(unsigned , unsigned , unsigned ) const;
   std::complex<double> CpconjUSubarCha2FdPL(unsigned gO1, unsigned gI1, unsigned gI2) const;
   std::complex<double> CpconjUSuconjRpmSd(unsigned gO2, unsigned gI1, unsigned gI2) const;
   std::complex<double> CpUSuconjUSuconjSvSv(unsigned gO1, unsigned gO2, unsigned gI1, unsigned gI2) const;
   std::complex<double> CpconjUSuFdCha1PR(unsigned gO2, unsigned gI1, unsigned gI2) const;
   std::complex<double> CpconjUSuFdCha1PL(unsigned gO1, unsigned gI1, unsigned gI2) const;
   std::complex<double> CpconjUSuFuChiPR(unsigned gO2, unsigned gI1, unsigned gI2) const;
   std::complex<double> CpconjUSuFuChiPL(unsigned gO1, unsigned gI1, unsigned gI2) const;
   std::complex<double> CpUSuconjUSuAhAh(unsigned gO1, unsigned gO2, unsigned gI1, unsigned gI2) const;
   std::complex<double> CpUSuconjUSuconjHpmHpm(unsigned gO1, unsigned gO2, unsigned gI1, unsigned gI2) const;
   std::complex<double> CpUSuconjUSuhhhh(unsigned gO1, unsigned gO2, unsigned gI1, unsigned gI2) const;
   std::complex<double> CpconjUSubarChiFuPR(unsigned gO2, unsigned gI1, unsigned gI2) const;
   std::complex<double> CpconjUSubarChiFuPL(unsigned gO1, unsigned gI1, unsigned gI2) const;
   std::complex<double> CpconjUSuconjHpmSd(unsigned gO2, unsigned gI1, unsigned gI2) const;
   std::complex<double> CpconjUSuSuSOc(unsigned gO2, unsigned gI1) const;
   std::complex<double> CpUSuconjUSuconjSdSd(unsigned gO1, unsigned gO2, unsigned gI1, unsigned gI2) const;
   std::complex<double> CpUSuconjUSuconjSeSe(unsigned gO1, unsigned gO2, unsigned gI1, unsigned gI2) const;
   std::complex<double> CpUSuconjUSuconjSuSu(unsigned gO1, unsigned gO2, unsigned gI1, unsigned gI2) const;
   std::complex<double> CpconjUSuSuRh(unsigned gO2, unsigned gI1, unsigned gI2) const;
   std::complex<double> CpconjUSuSuAh(unsigned gO2, unsigned gI1, unsigned gI2) const;
   std::complex<double> CpconjUSuSuhh(unsigned gO2, unsigned gI1, unsigned gI2) const;
   std::complex<double> CpconjUSubarGluFuPR(unsigned gO2, unsigned , unsigned gI2) const;
   double CpconjUSubarGluFuPL(unsigned , unsigned , unsigned ) const;
   double CpconjUSuGluFuPR(unsigned , unsigned , unsigned ) const;
   std::complex<double> CpconjUSuGluFuPL(unsigned gO1, unsigned , unsigned gI2) const;
   std::complex<double> CpconjUSuconjVWmSd(unsigned gO2, unsigned gI2) const;
   std::complex<double> CpconjUSuVGSu(unsigned gO2, unsigned gI2) const;
   std::complex<double> CpconjUSuVPSu(unsigned gO2, unsigned gI2) const;
   std::complex<double> CpconjUSuVZSu(unsigned gO2, unsigned gI2) const;
   std::complex<double> CpUSeconjUSeVZVZ(unsigned gO1, unsigned gO2) const;
   double CpUSeconjUSeconjVWmVWm(unsigned gO1, unsigned gO2) const;
   std::complex<double> CpUSeconjUSeconjRhRh(unsigned gO1, unsigned gO2, unsigned gI1, unsigned gI2) const;
   std::complex<double> CpUSeconjUSeconjRpmRpm(unsigned gO1, unsigned gO2, unsigned gI1, unsigned gI2) const;
   double CpconjUSebarCha1FvPR(unsigned , unsigned , unsigned ) const;
   std::complex<double> CpconjUSebarCha1FvPL(unsigned gO1, unsigned gI1, unsigned gI2) const;
   std::complex<double> CpUSeconjUSeconjSvSv(unsigned gO1, unsigned gO2, unsigned gI1, unsigned gI2) const;
   std::complex<double> CpconjUSeSvRpm(unsigned gO2, unsigned gI1, unsigned gI2) const;
   double CpconjUSeFvCha2PR(unsigned , unsigned , unsigned ) const;
   std::complex<double> CpconjUSeFvCha2PL(unsigned gO1, unsigned gI1, unsigned gI2) const;
   std::complex<double> CpconjUSeSvHpm(unsigned gO2, unsigned gI1, unsigned gI2) const;
   std::complex<double> CpconjUSeFeChiPR(unsigned gO2, unsigned gI1, unsigned gI2) const;
   std::complex<double> CpconjUSeFeChiPL(unsigned gO1, unsigned gI1, unsigned gI2) const;
   std::complex<double> CpUSeconjUSeAhAh(unsigned gO1, unsigned gO2, unsigned gI1, unsigned gI2) const;
   std::complex<double> CpUSeconjUSeconjHpmHpm(unsigned gO1, unsigned gO2, unsigned gI1, unsigned gI2) const;
   std::complex<double> CpUSeconjUSehhhh(unsigned gO1, unsigned gO2, unsigned gI1, unsigned gI2) const;
   std::complex<double> CpconjUSebarChiFePR(unsigned gO2, unsigned gI1, unsigned gI2) const;
   std::complex<double> CpconjUSebarChiFePL(unsigned gO1, unsigned gI1, unsigned gI2) const;
   std::complex<double> CpUSeconjUSeconjSdSd(unsigned gO1, unsigned gO2, unsigned gI1, unsigned gI2) const;
   std::complex<double> CpUSeconjUSeconjSeSe(unsigned gO1, unsigned gO2, unsigned gI1, unsigned gI2) const;
   std::complex<double> CpUSeconjUSeconjSuSu(unsigned gO1, unsigned gO2, unsigned gI1, unsigned gI2) const;
   std::complex<double> CpconjUSeSeRh(unsigned gO2, unsigned gI1, unsigned gI2) const;
   std::complex<double> CpconjUSeSeAh(unsigned gO2, unsigned gI1, unsigned gI2) const;
   std::complex<double> CpconjUSeSehh(unsigned gO2, unsigned gI1, unsigned gI2) const;
   std::complex<double> CpconjUSeVWmSv(unsigned gO2, unsigned gI2) const;
   std::complex<double> CpconjUSeVPSe(unsigned gO2, unsigned gI2) const;
   std::complex<double> CpconjUSeVZSe(unsigned gO2, unsigned gI2) const;
   std::complex<double> CpUhhVZVZ(unsigned gO2) const;
   std::complex<double> CpUhhconjVWmVWm(unsigned gO2) const;
   std::complex<double> CpUhhbargWmgWm(unsigned gO1) const;
   std::complex<double> CpUhhbargWmCgWmC(unsigned gO1) const;
   std::complex<double> CpUhhbargZgZ(unsigned gO1) const;
   std::complex<double> CpUhhUhhVZVZ(unsigned gO1, unsigned gO2) const;
   std::complex<double> CpUhhUhhconjVWmVWm(unsigned gO1, unsigned gO2) const;
   std::complex<double> CpUhhUhhconjRhRh(unsigned gO1, unsigned gO2, unsigned gI1, unsigned gI2) const;
   std::complex<double> CpUhhUhhconjRpmRpm(unsigned gO1, unsigned gO2, unsigned gI1, unsigned gI2) const;
   std::complex<double> CpUhhconjRhRh(unsigned gO2, unsigned gI1, unsigned gI2) const;
   std::complex<double> CpUhhconjRpmRpm(unsigned gO2, unsigned gI1, unsigned gI2) const;
   std::complex<double> CpUhhbarCha1Cha1PR(unsigned gO2, unsigned gI1, unsigned gI2) const;
   std::complex<double> CpUhhbarCha1Cha1PL(unsigned gO1, unsigned gI1, unsigned gI2) const;
   std::complex<double> CpUhhbarCha2Cha2PR(unsigned gO2, unsigned gI1, unsigned gI2) const;
   std::complex<double> CpUhhbarCha2Cha2PL(unsigned gO1, unsigned gI1, unsigned gI2) const;
   std::complex<double> CpUhhconjRhAh(unsigned gO2, unsigned gI1, unsigned gI2) const;
   std::complex<double> CpUhhconjRhhh(unsigned gO2, unsigned gI1, unsigned gI2) const;
   std::complex<double> CpUhhconjRpmHpm(unsigned gO2, unsigned gI1, unsigned gI2) const;
   std::complex<double> CpUhhUhhconjSvSv(unsigned gO1, unsigned gO2, unsigned gI1, unsigned gI2) const;
   std::complex<double> CpUhhconjSvSv(unsigned gO2, unsigned gI1, unsigned gI2) const;
   std::complex<double> CpUhhbarFdFdPR(unsigned gO2, unsigned gI1, unsigned gI2) const;
   std::complex<double> CpUhhbarFdFdPL(unsigned gO1, unsigned gI1, unsigned gI2) const;
   std::complex<double> CpUhhbarFeFePR(unsigned gO2, unsigned gI1, unsigned gI2) const;
   std::complex<double> CpUhhbarFeFePL(unsigned gO1, unsigned gI1, unsigned gI2) const;
   std::complex<double> CpUhhbarFuFuPR(unsigned gO2, unsigned gI1, unsigned gI2) const;
   std::complex<double> CpUhhbarFuFuPL(unsigned gO1, unsigned gI1, unsigned gI2) const;
   std::complex<double> CpUhhUhhAhAh(unsigned gO1, unsigned gO2, unsigned gI1, unsigned gI2) const;
   std::complex<double> CpUhhUhhconjHpmHpm(unsigned gO1, unsigned gO2, unsigned gI1, unsigned gI2) const;
   std::complex<double> CpUhhUhhhhhh(unsigned gO1, unsigned gO2, unsigned gI1, unsigned gI2) const;
   std::complex<double> CpUhhAhAh(unsigned gO2, unsigned gI1, unsigned gI2) const;
   std::complex<double> CpUhhconjHpmHpm(unsigned gO2, unsigned gI1, unsigned gI2) const;
   std::complex<double> CpUhhhhAh(unsigned gO2, unsigned gI1, unsigned gI2) const;
   std::complex<double> CpUhhhhhh(unsigned gO2, unsigned gI1, unsigned gI2) const;
   std::complex<double> CpUhhbarChiChiPR(unsigned gO2, unsigned gI1, unsigned gI2) const;
   std::complex<double> CpUhhbarChiChiPL(unsigned gO1, unsigned gI1, unsigned gI2) const;
   std::complex<double> CpUhhUhhconjSdSd(unsigned gO1, unsigned gO2, unsigned gI1, unsigned gI2) const;
   std::complex<double> CpUhhUhhconjSeSe(unsigned gO1, unsigned gO2, unsigned gI1, unsigned gI2) const;
   std::complex<double> CpUhhUhhconjSuSu(unsigned gO1, unsigned gO2, unsigned gI1, unsigned gI2) const;
   std::complex<double> CpUhhconjSdSd(unsigned gO2, unsigned gI1, unsigned gI2) const;
   std::complex<double> CpUhhconjSeSe(unsigned gO2, unsigned gI1, unsigned gI2) const;
   std::complex<double> CpUhhconjSuSu(unsigned gO2, unsigned gI1, unsigned gI2) const;
   std::complex<double> CpUhhVZAh(unsigned gO2, unsigned gI2) const;
   std::complex<double> CpUhhconjVWmHpm(unsigned gO2, unsigned gI2) const;
   std::complex<double> CpUAhbargWmgWm(unsigned gO1) const;
   std::complex<double> CpUAhbargWmCgWmC(unsigned gO1) const;
   std::complex<double> CpUAhUAhVZVZ(unsigned gO1, unsigned gO2) const;
   std::complex<double> CpUAhUAhconjVWmVWm(unsigned gO1, unsigned gO2) const;
   std::complex<double> CpUAhUAhconjRhRh(unsigned gO1, unsigned gO2, unsigned gI1, unsigned gI2) const;
   std::complex<double> CpUAhUAhconjRpmRpm(unsigned gO1, unsigned gO2, unsigned gI1, unsigned gI2) const;
   std::complex<double> CpUAhconjRhRh(unsigned gO2, unsigned gI1, unsigned gI2) const;
   std::complex<double> CpUAhconjRpmRpm(unsigned gO2, unsigned gI1, unsigned gI2) const;
   std::complex<double> CpUAhbarCha1Cha1PR(unsigned gO2, unsigned gI1, unsigned gI2) const;
   std::complex<double> CpUAhbarCha1Cha1PL(unsigned gO1, unsigned gI1, unsigned gI2) const;
   std::complex<double> CpUAhbarCha2Cha2PR(unsigned gO2, unsigned gI1, unsigned gI2) const;
   std::complex<double> CpUAhbarCha2Cha2PL(unsigned gO1, unsigned gI1, unsigned gI2) const;
   std::complex<double> CpUAhconjRhAh(unsigned gO2, unsigned gI1, unsigned gI2) const;
   std::complex<double> CpUAhconjRhhh(unsigned gO2, unsigned gI1, unsigned gI2) const;
   std::complex<double> CpUAhconjRpmHpm(unsigned gO2, unsigned gI1, unsigned gI2) const;
   std::complex<double> CpUAhUAhconjSvSv(unsigned gO1, unsigned gO2, unsigned gI1, unsigned gI2) const;
   std::complex<double> CpUAhconjSvSv(unsigned gO2, unsigned gI1, unsigned gI2) const;
   std::complex<double> CpUAhbarFdFdPR(unsigned gO2, unsigned gI1, unsigned gI2) const;
   std::complex<double> CpUAhbarFdFdPL(unsigned gO1, unsigned gI1, unsigned gI2) const;
   std::complex<double> CpUAhbarFeFePR(unsigned gO2, unsigned gI1, unsigned gI2) const;
   std::complex<double> CpUAhbarFeFePL(unsigned gO1, unsigned gI1, unsigned gI2) const;
   std::complex<double> CpUAhbarFuFuPR(unsigned gO2, unsigned gI1, unsigned gI2) const;
   std::complex<double> CpUAhbarFuFuPL(unsigned gO1, unsigned gI1, unsigned gI2) const;
   std::complex<double> CpUAhUAhAhAh(unsigned gO1, unsigned gO2, unsigned gI1, unsigned gI2) const;
   std::complex<double> CpUAhUAhconjHpmHpm(unsigned gO1, unsigned gO2, unsigned gI1, unsigned gI2) const;
   std::complex<double> CpUAhUAhhhhh(unsigned gO1, unsigned gO2, unsigned gI1, unsigned gI2) const;
   std::complex<double> CpUAhAhAh(unsigned gO2, unsigned gI1, unsigned gI2) const;
   std::complex<double> CpUAhconjHpmHpm(unsigned gO2, unsigned gI1, unsigned gI2) const;
   std::complex<double> CpUAhhhAh(unsigned gO2, unsigned gI1, unsigned gI2) const;
   std::complex<double> CpUAhhhhh(unsigned gO2, unsigned gI1, unsigned gI2) const;
   std::complex<double> CpUAhbarChiChiPR(unsigned gO2, unsigned gI1, unsigned gI2) const;
   std::complex<double> CpUAhbarChiChiPL(unsigned gO1, unsigned gI1, unsigned gI2) const;
   std::complex<double> CpUAhUAhconjSdSd(unsigned gO1, unsigned gO2, unsigned gI1, unsigned gI2) const;
   std::complex<double> CpUAhUAhconjSeSe(unsigned gO1, unsigned gO2, unsigned gI1, unsigned gI2) const;
   std::complex<double> CpUAhUAhconjSuSu(unsigned gO1, unsigned gO2, unsigned gI1, unsigned gI2) const;
   std::complex<double> CpUAhconjSdSd(unsigned gO2, unsigned gI1, unsigned gI2) const;
   std::complex<double> CpUAhconjSeSe(unsigned gO2, unsigned gI1, unsigned gI2) const;
   std::complex<double> CpUAhconjSuSu(unsigned gO2, unsigned gI1, unsigned gI2) const;
   std::complex<double> CpUAhVZhh(unsigned gO2, unsigned gI2) const;
   std::complex<double> CpUAhconjVWmHpm(unsigned gO2, unsigned gI2) const;
   std::complex<double> CpURhconjURhVZVZ(unsigned gO1, unsigned gO2) const;
   std::complex<double> CpURhconjURhconjVWmVWm(unsigned gO1, unsigned gO2) const;
   std::complex<double> CpURhconjURhconjRhRh(unsigned gO1, unsigned gO2, unsigned gI1, unsigned gI2) const;
   std::complex<double> CpURhconjURhconjRpmRpm(unsigned gO1, unsigned gO2, unsigned gI1, unsigned gI2) const;
   std::complex<double> CpconjURhconjRpmVWm(unsigned gO2, unsigned gI1) const;
   std::complex<double> CpconjURhCha2Cha1PR(unsigned gO2, unsigned gI1, unsigned gI2) const;
   std::complex<double> CpconjURhCha2Cha1PL(unsigned gO1, unsigned gI1, unsigned gI2) const;
   std::complex<double> CpconjURhconjRpmHpm(unsigned gO2, unsigned gI1, unsigned gI2) const;
   std::complex<double> CpconjURhRhAh(unsigned gO2, unsigned gI1, unsigned gI2) const;
   std::complex<double> CpconjURhRhhh(unsigned gO2, unsigned gI1, unsigned gI2) const;
   std::complex<double> CpURhconjURhconjSvSv(unsigned gO1, unsigned gO2, unsigned gI1, unsigned gI2) const;
   std::complex<double> CpURhconjURhAhAh(unsigned gO1, unsigned gO2, unsigned gI1, unsigned gI2) const;
   std::complex<double> CpURhconjURhconjHpmHpm(unsigned gO1, unsigned gO2, unsigned gI1, unsigned gI2) const;
   std::complex<double> CpURhconjURhhhhh(unsigned gO1, unsigned gO2, unsigned gI1, unsigned gI2) const;
   std::complex<double> CpconjURhconjHpmRpm(unsigned gO2, unsigned gI1, unsigned gI2) const;
   std::complex<double> CpconjURhAhAh(unsigned gO2, unsigned gI1, unsigned gI2) const;
   std::complex<double> CpconjURhconjHpmHpm(unsigned gO2, unsigned gI1, unsigned gI2) const;
   std::complex<double> CpconjURhhhAh(unsigned gO2, unsigned gI1, unsigned gI2) const;
   std::complex<double> CpconjURhhhhh(unsigned gO2, unsigned gI1, unsigned gI2) const;
   std::complex<double> CpconjURhChiChiPR(unsigned gO2, unsigned gI1, unsigned gI2) const;
   std::complex<double> CpconjURhChiChiPL(unsigned gO1, unsigned gI1, unsigned gI2) const;
   std::complex<double> CpURhconjURhconjSdSd(unsigned gO1, unsigned gO2, unsigned gI1, unsigned gI2) const;
   std::complex<double> CpURhconjURhconjSeSe(unsigned gO1, unsigned gO2, unsigned gI1, unsigned gI2) const;
   std::complex<double> CpURhconjURhconjSuSu(unsigned gO1, unsigned gO2, unsigned gI1, unsigned gI2) const;
   std::complex<double> CpconjURhconjSdSd(unsigned gO2, unsigned gI1, unsigned gI2) const;
   std::complex<double> CpconjURhconjSeSe(unsigned gO2, unsigned gI1, unsigned gI2) const;
   std::complex<double> CpconjURhconjSuSu(unsigned gO2, unsigned gI1, unsigned gI2) const;
   std::complex<double> CpconjURhVZRh(unsigned gO2, unsigned gI2) const;
   std::complex<double> CpconjURhconjVWmRpm(unsigned gO2, unsigned gI2) const;
   std::complex<double> CpconjUHpmVWmVP(unsigned gO2) const;
   std::complex<double> CpconjUHpmVZVWm(unsigned gO2) const;
   std::complex<double> CpconjUHpmbargWmCgZ(unsigned gO1) const;
   std::complex<double> CpUHpmgWmCbargZ(unsigned gO2) const;
   std::complex<double> CpconjUHpmbargZgWm(unsigned gO1) const;
   std::complex<double> CpUHpmgZbargWm(unsigned gO2) const;
   std::complex<double> CpUHpmconjUHpmVZVZ(unsigned gO1, unsigned gO2) const;
   std::complex<double> CpUHpmconjUHpmconjVWmVWm(unsigned gO1, unsigned gO2) const;
   std::complex<double> CpUHpmconjUHpmconjRhRh(unsigned gO1, unsigned gO2, unsigned gI1, unsigned gI2) const;
   std::complex<double> CpUHpmconjUHpmconjRpmRpm(unsigned gO1, unsigned gO2, unsigned gI1, unsigned gI2) const;
   std::complex<double> CpconjUHpmconjRhRpm(unsigned gO2, unsigned gI1, unsigned gI2) const;
   std::complex<double> CpconjUHpmRpmRh(unsigned gO2, unsigned gI1, unsigned gI2) const;
   std::complex<double> CpconjUHpmconjRhHpm(unsigned gO2, unsigned gI1, unsigned gI2) const;
   std::complex<double> CpconjUHpmRpmAh(unsigned gO2, unsigned gI1, unsigned gI2) const;
   std::complex<double> CpconjUHpmRpmhh(unsigned gO2, unsigned gI1, unsigned gI2) const;
   std::complex<double> CpconjUHpmbarCha1ChiPR(unsigned gO2, unsigned gI1, unsigned gI2) const;
   std::complex<double> CpconjUHpmbarCha1ChiPL(unsigned gO1, unsigned gI1, unsigned gI2) const;
   std::complex<double> CpUHpmconjUHpmconjSvSv(unsigned gO1, unsigned gO2, unsigned gI1, unsigned gI2) const;
   std::complex<double> CpconjUHpmbarFuFdPR(unsigned gO2, unsigned gI1, unsigned gI2) const;
   std::complex<double> CpconjUHpmbarFuFdPL(unsigned gO1, unsigned gI1, unsigned gI2) const;
   std::complex<double> CpconjUHpmbarFvFePR(unsigned gO2, unsigned gI1, unsigned gI2) const;
   double CpconjUHpmbarFvFePL(unsigned , unsigned , unsigned ) const;
   std::complex<double> CpconjUHpmconjSvSe(unsigned gO2, unsigned gI1, unsigned gI2) const;
   std::complex<double> CpUHpmconjUHpmAhAh(unsigned gO1, unsigned gO2, unsigned gI1, unsigned gI2) const;
   std::complex<double> CpUHpmconjUHpmconjHpmHpm(unsigned gO1, unsigned gO2, unsigned gI1, unsigned gI2) const;
   std::complex<double> CpUHpmconjUHpmhhhh(unsigned gO1, unsigned gO2, unsigned gI1, unsigned gI2) const;
   std::complex<double> CpconjUHpmbarChiCha2PR(unsigned gO2, unsigned gI1, unsigned gI2) const;
   std::complex<double> CpconjUHpmbarChiCha2PL(unsigned gO1, unsigned gI1, unsigned gI2) const;
   std::complex<double> CpconjUHpmHpmAh(unsigned gO2, unsigned gI1, unsigned gI2) const;
   std::complex<double> CpconjUHpmHpmhh(unsigned gO2, unsigned gI1, unsigned gI2) const;
   std::complex<double> CpUHpmconjUHpmconjSdSd(unsigned gO1, unsigned gO2, unsigned gI1, unsigned gI2) const;
   std::complex<double> CpUHpmconjUHpmconjSeSe(unsigned gO1, unsigned gO2, unsigned gI1, unsigned gI2) const;
   std::complex<double> CpUHpmconjUHpmconjSuSu(unsigned gO1, unsigned gO2, unsigned gI1, unsigned gI2) const;
   std::complex<double> CpconjUHpmconjSuSd(unsigned gO2, unsigned gI1, unsigned gI2) const;
   std::complex<double> CpconjUHpmVWmAh(unsigned gO2, unsigned gI2) const;
   std::complex<double> CpconjUHpmVWmhh(unsigned gO2, unsigned gI2) const;
   std::complex<double> CpconjUHpmVPHpm(unsigned gO2, unsigned gI2) const;
   std::complex<double> CpconjUHpmVZHpm(unsigned gO2, unsigned gI2) const;
   std::complex<double> CpURpmconjURpmVZVZ(unsigned gO1, unsigned gO2) const;
   std::complex<double> CpURpmconjURpmconjVWmVWm(unsigned gO1, unsigned gO2) const;
   std::complex<double> CpURpmconjURpmconjRhRh(unsigned gO1, unsigned gO2, unsigned gI1, unsigned gI2) const;
   std::complex<double> CpURpmconjURpmconjRpmRpm(unsigned gO1, unsigned gO2, unsigned gI1, unsigned gI2) const;
   std::complex<double> CpconjURpmconjRhVWm(unsigned gO2, unsigned gI1) const;
   std::complex<double> CpconjURpmconjRhHpm(unsigned gO2, unsigned gI1, unsigned gI2) const;
   std::complex<double> CpconjURpmRhHpm(unsigned gO2, unsigned gI1, unsigned gI2) const;
   std::complex<double> CpconjURpmRpmAh(unsigned gO2, unsigned gI1, unsigned gI2) const;
   std::complex<double> CpconjURpmRpmhh(unsigned gO2, unsigned gI1, unsigned gI2) const;
   std::complex<double> CpURpmconjURpmconjSvSv(unsigned gO1, unsigned gO2, unsigned gI1, unsigned gI2) const;
   std::complex<double> CpconjURpmconjSvSe(unsigned gO2, unsigned gI1, unsigned gI2) const;
   std::complex<double> CpURpmconjURpmAhAh(unsigned gO1, unsigned gO2, unsigned gI1, unsigned gI2) const;
   std::complex<double> CpURpmconjURpmconjHpmHpm(unsigned gO1, unsigned gO2, unsigned gI1, unsigned gI2) const;
   std::complex<double> CpURpmconjURpmhhhh(unsigned gO1, unsigned gO2, unsigned gI1, unsigned gI2) const;
   std::complex<double> CpconjURpmbarChibarCha1PR(unsigned gO2, unsigned gI1, unsigned gI2) const;
   std::complex<double> CpconjURpmbarChibarCha1PL(unsigned gO1, unsigned gI1, unsigned gI2) const;
   std::complex<double> CpconjURpmChiCha2PR(unsigned gO2, unsigned gI1, unsigned gI2) const;
   std::complex<double> CpconjURpmChiCha2PL(unsigned gO1, unsigned gI1, unsigned gI2) const;
   std::complex<double> CpconjURpmHpmAh(unsigned gO2, unsigned gI1, unsigned gI2) const;
   std::complex<double> CpconjURpmHpmhh(unsigned gO2, unsigned gI1, unsigned gI2) const;
   std::complex<double> CpURpmconjURpmconjSdSd(unsigned gO1, unsigned gO2, unsigned gI1, unsigned gI2) const;
   std::complex<double> CpURpmconjURpmconjSeSe(unsigned gO1, unsigned gO2, unsigned gI1, unsigned gI2) const;
   std::complex<double> CpURpmconjURpmconjSuSu(unsigned gO1, unsigned gO2, unsigned gI1, unsigned gI2) const;
   std::complex<double> CpconjURpmconjSuSd(unsigned gO2, unsigned gI1, unsigned gI2) const;
   std::complex<double> CpconjURpmVWmRh(unsigned gO2, unsigned gI2) const;
   std::complex<double> CpconjURpmVPRpm(unsigned gO2, unsigned gI2) const;
   std::complex<double> CpconjURpmVZRpm(unsigned gO2, unsigned gI2) const;
   double CpSOcconjSOcconjSOcSOc() const;
   std::complex<double> CpconjSOcVGSOc() const;
   double CpSOcconjSOcconjSdSd(unsigned , unsigned ) const;
   double CpSOcconjSOcconjSuSu(unsigned , unsigned ) const;
   std::complex<double> CpconjSOcconjSdSd(unsigned gI1, unsigned gI2) const;
   std::complex<double> CpconjSOcconjSuSu(unsigned gI1, unsigned gI2) const;
   double CpconjSOcbarGluGluPR(unsigned , unsigned ) const;
   std::complex<double> CpconjSOcbarGluGluPL(unsigned , unsigned ) const;
   double CpVZbargWmgWm() const;
   double CpVZbargWmCgWmC() const;
   double CpVZconjVWmVWm() const;
   std::complex<double> CpVZVZconjRhRh(unsigned gI1, unsigned gI2) const;
   std::complex<double> CpVZVZconjRpmRpm(unsigned gI1, unsigned gI2) const;
   std::complex<double> CpVZconjRhRh(unsigned gI1, unsigned gI2) const;
   double CpVZconjRpmRpm(unsigned gI1, unsigned gI2) const;
   std::complex<double> CpVZbarCha1Cha1PL(unsigned gI1, unsigned gI2) const;
   std::complex<double> CpVZbarCha1Cha1PR(unsigned gI1, unsigned gI2) const;
   std::complex<double> CpVZbarCha2Cha2PL(unsigned gI1, unsigned gI2) const;
   std::complex<double> CpVZbarCha2Cha2PR(unsigned gI1, unsigned gI2) const;
   std::complex<double> CpVZVZconjSvSv(unsigned gI1, unsigned gI2) const;
   double CpVZconjSvSv(unsigned gI1, unsigned gI2) const;
   double CpVZbarFdFdPL(unsigned gI1, unsigned gI2) const;
   double CpVZbarFdFdPR(unsigned gI1, unsigned gI2) const;
   double CpVZbarFeFePL(unsigned gI1, unsigned gI2) const;
   double CpVZbarFeFePR(unsigned gI1, unsigned gI2) const;
   double CpVZbarFuFuPL(unsigned gI1, unsigned gI2) const;
   double CpVZbarFuFuPR(unsigned gI1, unsigned gI2) const;
   double CpVZbarFvFvPL(unsigned gI1, unsigned gI2) const;
   double CpVZbarFvFvPR(unsigned , unsigned ) const;
   std::complex<double> CpVZVZAhAh(unsigned gI1, unsigned gI2) const;
   std::complex<double> CpVZVZconjHpmHpm(unsigned gI1, unsigned gI2) const;
   std::complex<double> CpVZVZhhhh(unsigned gI1, unsigned gI2) const;
   std::complex<double> CpVZconjHpmHpm(unsigned gI1, unsigned gI2) const;
   std::complex<double> CpVZhhAh(unsigned gI1, unsigned gI2) const;
   std::complex<double> CpVZbarChiChiPL(unsigned gI1, unsigned gI2) const;
   std::complex<double> CpVZbarChiChiPR(unsigned gI1, unsigned gI2) const;
   std::complex<double> CpVZVZconjSdSd(unsigned gI1, unsigned gI2) const;
   std::complex<double> CpVZVZconjSeSe(unsigned gI1, unsigned gI2) const;
   std::complex<double> CpVZVZconjSuSu(unsigned gI1, unsigned gI2) const;
   std::complex<double> CpVZconjSdSd(unsigned gI1, unsigned gI2) const;
   std::complex<double> CpVZconjSeSe(unsigned gI1, unsigned gI2) const;
   std::complex<double> CpVZconjSuSu(unsigned gI1, unsigned gI2) const;
   std::complex<double> CpVZVZhh(unsigned gI2) const;
   std::complex<double> CpVZconjVWmHpm(unsigned gI2) const;
   double CpVZVZconjVWmVWm1() const;
   double CpVZVZconjVWmVWm2() const;
   double CpVZVZconjVWmVWm3() const;
   double CpconjVWmbargPgWm() const;
   double CpconjVWmbargWmCgP() const;
   double CpconjVWmbargWmCgZ() const;
   double CpconjVWmbargZgWm() const;
   double CpconjVWmVWmVP() const;
   double CpconjVWmVZVWm() const;
   std::complex<double> CpVWmconjVWmconjRhRh(unsigned gI1, unsigned gI2) const;
   std::complex<double> CpVWmconjVWmconjRpmRpm(unsigned gI1, unsigned gI2) const;
   std::complex<double> CpconjVWmconjRhRpm(unsigned gI1, unsigned gI2) const;
   std::complex<double> CpconjVWmRpmRh(unsigned gI1, unsigned gI2) const;
   std::complex<double> CpconjVWmbarCha1ChiPL(unsigned gI1, unsigned gI2) const;
   std::complex<double> CpconjVWmbarCha1ChiPR(unsigned gI1, unsigned gI2) const;
   double CpVWmconjVWmconjSvSv(unsigned gI1, unsigned gI2) const;
   std::complex<double> CpconjVWmbarFuFdPL(unsigned gI1, unsigned gI2) const;
   double CpconjVWmbarFuFdPR(unsigned , unsigned ) const;
   std::complex<double> CpconjVWmbarFvFePL(unsigned gI1, unsigned gI2) const;
   double CpconjVWmbarFvFePR(unsigned , unsigned ) const;
   std::complex<double> CpconjVWmconjSvSe(unsigned gI1, unsigned gI2) const;
   std::complex<double> CpVWmconjVWmAhAh(unsigned gI1, unsigned gI2) const;
   std::complex<double> CpVWmconjVWmconjHpmHpm(unsigned gI1, unsigned gI2) const;
   std::complex<double> CpVWmconjVWmhhhh(unsigned gI1, unsigned gI2) const;
   std::complex<double> CpconjVWmbarChiCha2PL(unsigned gI1, unsigned gI2) const;
   std::complex<double> CpconjVWmbarChiCha2PR(unsigned gI1, unsigned gI2) const;
   std::complex<double> CpconjVWmHpmAh(unsigned gI1, unsigned gI2) const;
   std::complex<double> CpconjVWmHpmhh(unsigned gI1, unsigned gI2) const;
   std::complex<double> CpVWmconjVWmconjSdSd(unsigned gI1, unsigned gI2) const;
   std::complex<double> CpVWmconjVWmconjSeSe(unsigned gI1, unsigned gI2) const;
   std::complex<double> CpVWmconjVWmconjSuSu(unsigned gI1, unsigned gI2) const;
   std::complex<double> CpconjVWmconjSuSd(unsigned gI1, unsigned gI2) const;
   std::complex<double> CpconjVWmVPHpm(unsigned gI2) const;
   std::complex<double> CpconjVWmVWmhh(unsigned gI2) const;
   std::complex<double> CpconjVWmVZHpm(unsigned gI2) const;
   double CpVWmconjVWmVPVP1() const;
   double CpVWmconjVWmVPVP2() const;
   double CpVWmconjVWmVPVP3() const;
   double CpVWmconjVWmVZVZ1() const;
   double CpVWmconjVWmVZVZ2() const;
   double CpVWmconjVWmVZVZ3() const;
   double CpVWmconjVWmconjVWmVWm1() const;
   double CpVWmconjVWmconjVWmVWm2() const;
   double CpVWmconjVWmconjVWmVWm3() const;
   std::complex<double> CpbarUChibarCha2RpmPL(unsigned gO2, unsigned gI1, unsigned gI2) const;
   std::complex<double> CpbarUChibarCha2RpmPR(unsigned gO1, unsigned gI1, unsigned gI2) const;
   std::complex<double> CpbarUChiconjRpmbarCha1PL(unsigned gO2, unsigned gI1, unsigned gI2) const;
   std::complex<double> CpbarUChiconjRpmbarCha1PR(unsigned gO1, unsigned gI1, unsigned gI2) const;
   double CpbarUChibarFvSvPL(unsigned , unsigned , unsigned ) const;
   std::complex<double> CpbarUChibarFvSvPR(unsigned gO1, unsigned gI1, unsigned gI2) const;
   std::complex<double> CpbarUChibarFdSdPL(unsigned gO2, unsigned gI1, unsigned gI2) const;
   std::complex<double> CpbarUChibarFdSdPR(unsigned gO1, unsigned gI1, unsigned gI2) const;
   std::complex<double> CpbarUChibarFeSePL(unsigned gO2, unsigned gI1, unsigned gI2) const;
   std::complex<double> CpbarUChibarFeSePR(unsigned gO1, unsigned gI1, unsigned gI2) const;
   std::complex<double> CpbarUChibarFuSuPL(unsigned gO2, unsigned gI1, unsigned gI2) const;
   std::complex<double> CpbarUChibarFuSuPR(unsigned gO1, unsigned gI1, unsigned gI2) const;
   std::complex<double> CpbarUChibarChiRhPL(unsigned gO2, unsigned gI1, unsigned gI2) const;
   std::complex<double> CpbarUChibarChiRhPR(unsigned gO1, unsigned gI1, unsigned gI2) const;
   std::complex<double> CpbarUChiconjHpmCha2PL(unsigned gO2, unsigned gI1, unsigned gI2) const;
   std::complex<double> CpbarUChiconjHpmCha2PR(unsigned gO1, unsigned gI1, unsigned gI2) const;
   std::complex<double> CpbarUChiHpmCha1PL(unsigned gO2, unsigned gI1, unsigned gI2) const;
   std::complex<double> CpbarUChiHpmCha1PR(unsigned gO1, unsigned gI1, unsigned gI2) const;
   std::complex<double> CpbarUChiChiAhPL(unsigned gO2, unsigned gI1, unsigned gI2) const;
   std::complex<double> CpbarUChiChiAhPR(unsigned gO1, unsigned gI1, unsigned gI2) const;
   std::complex<double> CpbarUChihhChiPL(unsigned gO2, unsigned gI1, unsigned gI2) const;
   std::complex<double> CpbarUChihhChiPR(unsigned gO1, unsigned gI1, unsigned gI2) const;
   std::complex<double> CpbarUChiconjSdFdPL(unsigned gO2, unsigned gI1, unsigned gI2) const;
   std::complex<double> CpbarUChiconjSdFdPR(unsigned gO1, unsigned gI1, unsigned gI2) const;
   std::complex<double> CpbarUChiconjSeFePL(unsigned gO2, unsigned gI1, unsigned gI2) const;
   std::complex<double> CpbarUChiconjSeFePR(unsigned gO1, unsigned gI1, unsigned gI2) const;
   std::complex<double> CpbarUChiconjSuFuPL(unsigned gO2, unsigned gI1, unsigned gI2) const;
   std::complex<double> CpbarUChiconjSuFuPR(unsigned gO1, unsigned gI1, unsigned gI2) const;
   std::complex<double> CpbarUChiVWmCha1PR(unsigned gO2, unsigned gI2) const;
   std::complex<double> CpbarUChiVWmCha1PL(unsigned gO1, unsigned gI2) const;
   std::complex<double> CpbarUChiconjVWmCha2PR(unsigned gO2, unsigned gI2) const;
   std::complex<double> CpbarUChiconjVWmCha2PL(unsigned gO1, unsigned gI2) const;
   std::complex<double> CpbarUChiVZChiPR(unsigned gO2, unsigned gI2) const;
   std::complex<double> CpbarUChiVZChiPL(unsigned gO1, unsigned gI2) const;
   std::complex<double> CpbarUCha1barCha2RhPL(unsigned gO2, unsigned gI1, unsigned gI2) const;
   std::complex<double> CpbarUCha1barCha2RhPR(unsigned gO1, unsigned gI1, unsigned gI2) const;
   std::complex<double> CpbarUCha1Cha1AhPL(unsigned gO2, unsigned gI1, unsigned gI2) const;
   std::complex<double> CpbarUCha1Cha1AhPR(unsigned gO1, unsigned gI1, unsigned gI2) const;
   std::complex<double> CpbarUCha1conjRpmbarChiPL(unsigned gO2, unsigned gI1, unsigned gI2) const;
   std::complex<double> CpbarUCha1conjRpmbarChiPR(unsigned gO1, unsigned gI1, unsigned gI2) const;
   std::complex<double> CpbarUCha1barFeSvPL(unsigned gO2, unsigned gI1, unsigned gI2) const;
   std::complex<double> CpbarUCha1barFeSvPR(unsigned gO1, unsigned gI1, unsigned gI2) const;
   std::complex<double> CpbarUCha1barFdSuPL(unsigned gO2, unsigned gI1, unsigned gI2) const;
   std::complex<double> CpbarUCha1barFdSuPR(unsigned gO1, unsigned gI1, unsigned gI2) const;
   std::complex<double> CpbarUCha1hhCha1PL(unsigned gO2, unsigned gI1, unsigned gI2) const;
   std::complex<double> CpbarUCha1hhCha1PR(unsigned gO1, unsigned gI1, unsigned gI2) const;
   std::complex<double> CpbarUCha1conjHpmChiPL(unsigned gO2, unsigned gI1, unsigned gI2) const;
   std::complex<double> CpbarUCha1conjHpmChiPR(unsigned gO1, unsigned gI1, unsigned gI2) const;
   std::complex<double> CpbarUCha1conjSdFuPL(unsigned gO2, unsigned gI1, unsigned gI2) const;
   double CpbarUCha1conjSdFuPR(unsigned , unsigned , unsigned ) const;
   std::complex<double> CpbarUCha1conjSeFvPL(unsigned gO2, unsigned gI1, unsigned gI2) const;
   double CpbarUCha1conjSeFvPR(unsigned , unsigned , unsigned ) const;
   std::complex<double> CpbarUCha1VPCha1PR(unsigned gO2, unsigned gI2) const;
   std::complex<double> CpbarUCha1VPCha1PL(unsigned gO1, unsigned gI2) const;
   std::complex<double> CpbarUCha1VZCha1PR(unsigned gO2, unsigned gI2) const;
   std::complex<double> CpbarUCha1VZCha1PL(unsigned gO1, unsigned gI2) const;
   std::complex<double> CpbarUCha1conjVWmChiPR(unsigned gO2, unsigned gI2) const;
   std::complex<double> CpbarUCha1conjVWmChiPL(unsigned gO1, unsigned gI2) const;
   std::complex<double> CpbarUCha2barCha1RhPL(unsigned gO2, unsigned gI1, unsigned gI2) const;
   std::complex<double> CpbarUCha2barCha1RhPR(unsigned gO1, unsigned gI1, unsigned gI2) const;
   std::complex<double> CpbarUCha2Cha2AhPL(unsigned gO2, unsigned gI1, unsigned gI2) const;
   std::complex<double> CpbarUCha2Cha2AhPR(unsigned gO1, unsigned gI1, unsigned gI2) const;
   std::complex<double> CpbarUCha2barFuSdPL(unsigned gO2, unsigned gI1, unsigned gI2) const;
   std::complex<double> CpbarUCha2barFuSdPR(unsigned gO1, unsigned gI1, unsigned gI2) const;
   double CpbarUCha2barFvSePL(unsigned , unsigned , unsigned ) const;
   std::complex<double> CpbarUCha2barFvSePR(unsigned gO1, unsigned gI1, unsigned gI2) const;
   std::complex<double> CpbarUCha2barChiRpmPL(unsigned gO2, unsigned gI1, unsigned gI2) const;
   std::complex<double> CpbarUCha2barChiRpmPR(unsigned gO1, unsigned gI1, unsigned gI2) const;
   std::complex<double> CpbarUCha2hhCha2PL(unsigned gO2, unsigned gI1, unsigned gI2) const;
   std::complex<double> CpbarUCha2hhCha2PR(unsigned gO1, unsigned gI1, unsigned gI2) const;
   std::complex<double> CpbarUCha2HpmChiPL(unsigned gO2, unsigned gI1, unsigned gI2) const;
   std::complex<double> CpbarUCha2HpmChiPR(unsigned gO1, unsigned gI1, unsigned gI2) const;
   std::complex<double> CpbarUCha2conjSuFdPL(unsigned gO2, unsigned gI1, unsigned gI2) const;
   double CpbarUCha2conjSuFdPR(unsigned , unsigned , unsigned ) const;
   std::complex<double> CpbarUCha2VPCha2PR(unsigned gO2, unsigned gI2) const;
   std::complex<double> CpbarUCha2VPCha2PL(unsigned gO1, unsigned gI2) const;
   std::complex<double> CpbarUCha2VZCha2PR(unsigned gO2, unsigned gI2) const;
   std::complex<double> CpbarUCha2VZCha2PL(unsigned gO1, unsigned gI2) const;
   std::complex<double> CpbarUCha2VWmChiPR(unsigned gO2, unsigned gI2) const;
   std::complex<double> CpbarUCha2VWmChiPL(unsigned gO1, unsigned gI2) const;
   std::complex<double> CpbarUFebarCha1SvPL(unsigned gO2, unsigned gI1, unsigned gI2) const;
   std::complex<double> CpbarUFebarCha1SvPR(unsigned gO1, unsigned gI1, unsigned gI2) const;
   std::complex<double> CpbarUFeFeAhPL(unsigned gO2, unsigned gI1, unsigned gI2) const;
   std::complex<double> CpbarUFeFeAhPR(unsigned gO1, unsigned gI1, unsigned gI2) const;
   std::complex<double> CpbarUFehhFePL(unsigned gO2, unsigned gI1, unsigned gI2) const;
   std::complex<double> CpbarUFehhFePR(unsigned gO1, unsigned gI1, unsigned gI2) const;
   std::complex<double> CpbarUFeHpmFvPL(unsigned gO2, unsigned gI1, unsigned gI2) const;
   double CpbarUFeHpmFvPR(unsigned , unsigned , unsigned ) const;
   std::complex<double> CpbarUFebarChiSePL(unsigned gO2, unsigned gI1, unsigned gI2) const;
   std::complex<double> CpbarUFebarChiSePR(unsigned gO1, unsigned gI1, unsigned gI2) const;
   std::complex<double> CpbarUFeSeChiPL(unsigned gO2, unsigned gI1, unsigned gI2) const;
   std::complex<double> CpbarUFeSeChiPR(unsigned gO1, unsigned gI1, unsigned gI2) const;
   std::complex<double> CpbarUFeVPFePR(unsigned gO2, unsigned gI2) const;
   std::complex<double> CpbarUFeVPFePL(unsigned gO1, unsigned gI2) const;
   double CpbarUFeVWmFvPR(unsigned , unsigned ) const;
   double CpbarUFeVWmFvPL(unsigned gO1, unsigned gI2) const;
   std::complex<double> CpbarUFeVZFePR(unsigned gO2, unsigned gI2) const;
   std::complex<double> CpbarUFeVZFePL(unsigned gO1, unsigned gI2) const;
   std::complex<double> CpbarUFdbarCha1SuPL(unsigned gO2, unsigned gI1, unsigned gI2) const;
   std::complex<double> CpbarUFdbarCha1SuPR(unsigned gO1, unsigned gI1, unsigned gI2) const;
   std::complex<double> CpbarUFdFdAhPL(unsigned gO2, unsigned gI1, unsigned gI2) const;
   std::complex<double> CpbarUFdFdAhPR(unsigned gO1, unsigned gI1, unsigned gI2) const;
   std::complex<double> CpbarUFdhhFdPL(unsigned gO2, unsigned gI1, unsigned gI2) const;
   std::complex<double> CpbarUFdhhFdPR(unsigned gO1, unsigned gI1, unsigned gI2) const;
   std::complex<double> CpbarUFdHpmFuPL(unsigned gO2, unsigned gI1, unsigned gI2) const;
   std::complex<double> CpbarUFdHpmFuPR(unsigned gO1, unsigned gI1, unsigned gI2) const;
   std::complex<double> CpbarUFdbarChiSdPL(unsigned gO2, unsigned gI1, unsigned gI2) const;
   std::complex<double> CpbarUFdbarChiSdPR(unsigned gO1, unsigned gI1, unsigned gI2) const;
   double CpbarUFdSuCha2PL(unsigned , unsigned , unsigned ) const;
   std::complex<double> CpbarUFdSuCha2PR(unsigned gO1, unsigned gI1, unsigned gI2) const;
   std::complex<double> CpbarUFdSdChiPL(unsigned gO2, unsigned gI1, unsigned gI2) const;
   std::complex<double> CpbarUFdSdChiPR(unsigned gO1, unsigned gI1, unsigned gI2) const;
   std::complex<double> CpbarUFdSdGluPL(unsigned gO2, unsigned gI1, unsigned ) const;
   double CpbarUFdSdGluPR(unsigned , unsigned , unsigned ) const;
   std::complex<double> CpbarUFdVGFdPR(unsigned gO2, unsigned gI2) const;
   std::complex<double> CpbarUFdVGFdPL(unsigned gO1, unsigned gI2) const;
   std::complex<double> CpbarUFdVPFdPR(unsigned gO2, unsigned gI2) const;
   std::complex<double> CpbarUFdVPFdPL(unsigned gO1, unsigned gI2) const;
   double CpbarUFdVWmFuPR(unsigned , unsigned ) const;
   std::complex<double> CpbarUFdVWmFuPL(unsigned gO1, unsigned gI2) const;
   std::complex<double> CpbarUFdVZFdPR(unsigned gO2, unsigned gI2) const;
   std::complex<double> CpbarUFdVZFdPL(unsigned gO1, unsigned gI2) const;
   double CpbarUFdbarGluSdPL(unsigned , unsigned , unsigned ) const;
   std::complex<double> CpbarUFdbarGluSdPR(unsigned gO1, unsigned , unsigned gI2) const;
   std::complex<double> CpbarUFubarCha2SdPL(unsigned gO2, unsigned gI1, unsigned gI2) const;
   std::complex<double> CpbarUFubarCha2SdPR(unsigned gO1, unsigned gI1, unsigned gI2) const;
   std::complex<double> CpbarUFuFuAhPL(unsigned gO2, unsigned gI1, unsigned gI2) const;
   std::complex<double> CpbarUFuFuAhPR(unsigned gO1, unsigned gI1, unsigned gI2) const;
   std::complex<double> CpbarUFuconjHpmFdPL(unsigned gO2, unsigned gI1, unsigned gI2) const;
   std::complex<double> CpbarUFuconjHpmFdPR(unsigned gO1, unsigned gI1, unsigned gI2) const;
   std::complex<double> CpbarUFuhhFuPL(unsigned gO2, unsigned gI1, unsigned gI2) const;
   std::complex<double> CpbarUFuhhFuPR(unsigned gO1, unsigned gI1, unsigned gI2) const;
   std::complex<double> CpbarUFubarChiSuPL(unsigned gO2, unsigned gI1, unsigned gI2) const;
   std::complex<double> CpbarUFubarChiSuPR(unsigned gO1, unsigned gI1, unsigned gI2) const;
   double CpbarUFuSdCha1PL(unsigned , unsigned , unsigned ) const;
   std::complex<double> CpbarUFuSdCha1PR(unsigned gO1, unsigned gI1, unsigned gI2) const;
   std::complex<double> CpbarUFuSuChiPL(unsigned gO2, unsigned gI1, unsigned gI2) const;
   std::complex<double> CpbarUFuSuChiPR(unsigned gO1, unsigned gI1, unsigned gI2) const;
   std::complex<double> CpbarUFuSuGluPL(unsigned gO2, unsigned gI1, unsigned ) const;
   double CpbarUFuSuGluPR(unsigned , unsigned , unsigned ) const;
   std::complex<double> CpbarUFuVGFuPR(unsigned gO2, unsigned gI2) const;
   std::complex<double> CpbarUFuVGFuPL(unsigned gO1, unsigned gI2) const;
   std::complex<double> CpbarUFuVPFuPR(unsigned gO2, unsigned gI2) const;
   std::complex<double> CpbarUFuVPFuPL(unsigned gO1, unsigned gI2) const;
   std::complex<double> CpbarUFuVZFuPR(unsigned gO2, unsigned gI2) const;
   std::complex<double> CpbarUFuVZFuPL(unsigned gO1, unsigned gI2) const;
   double CpbarUFuconjVWmFdPR(unsigned , unsigned ) const;
   std::complex<double> CpbarUFuconjVWmFdPL(unsigned gO1, unsigned gI2) const;
   double CpbarUFubarGluSuPL(unsigned , unsigned , unsigned ) const;
   std::complex<double> CpbarUFubarGluSuPR(unsigned gO1, unsigned , unsigned gI2) const;
   double CpbarGlubarFdSdPL(unsigned , unsigned ) const;
   std::complex<double> CpbarGlubarFdSdPR(unsigned gI1, unsigned gI2) const;
   double CpbarGlubarFuSuPL(unsigned , unsigned ) const;
   std::complex<double> CpbarGlubarFuSuPR(unsigned gI1, unsigned gI2) const;
   double CpbarGluconjSdFdPL(unsigned , unsigned ) const;
   std::complex<double> CpbarGluconjSdFdPR(unsigned gI1, unsigned gI2) const;
   double CpbarGluconjSuFuPL(unsigned , unsigned ) const;
   std::complex<double> CpbarGluconjSuFuPR(unsigned gI1, unsigned gI2) const;
   std::complex<double> CpbarGluVGGluPR() const;
   std::complex<double> CpbarGluVGGluPL() const;
   std::complex<double> CpbarGluconjSOcGluPL() const;
   double CpbarGluconjSOcGluPR() const;
   std::complex<double> CpbarFebarCha1SvPL(unsigned gO2, unsigned gI1, unsigned gI2) const;
   std::complex<double> CpbarFebarCha1SvPR(unsigned gO1, unsigned gI1, unsigned gI2) const;
   std::complex<double> CpbarFeFeAhPL(unsigned gO2, unsigned gI1, unsigned gI2) const;
   std::complex<double> CpbarFeFeAhPR(unsigned gO1, unsigned gI1, unsigned gI2) const;
   std::complex<double> CpbarFehhFePL(unsigned gO2, unsigned gI1, unsigned gI2) const;
   std::complex<double> CpbarFehhFePR(unsigned gO1, unsigned gI1, unsigned gI2) const;
   std::complex<double> CpbarFeHpmFvPL(unsigned gO2, unsigned gI1, unsigned gI2) const;
   double CpbarFeHpmFvPR(unsigned , unsigned , unsigned ) const;
   std::complex<double> CpbarFebarChiSePL(unsigned gO2, unsigned gI1, unsigned gI2) const;
   std::complex<double> CpbarFebarChiSePR(unsigned gO1, unsigned gI1, unsigned gI2) const;
   std::complex<double> CpbarFeSeChiPL(unsigned gO2, unsigned gI1, unsigned gI2) const;
   std::complex<double> CpbarFeSeChiPR(unsigned gO1, unsigned gI1, unsigned gI2) const;
   double CpbarFeVWmFvPR(unsigned , unsigned ) const;
   std::complex<double> CpbarFeVWmFvPL(unsigned gO1, unsigned gI2) const;
   double CpbarFeVZFePR(unsigned gO2, unsigned gI2) const;
   double CpbarFeVZFePL(unsigned gO1, unsigned gI2) const;
   std::complex<double> CpbarFdbarCha1SuPL(unsigned gO2, unsigned gI1, unsigned gI2) const;
   std::complex<double> CpbarFdbarCha1SuPR(unsigned gO1, unsigned gI1, unsigned gI2) const;
   std::complex<double> CpbarFdFdAhPL(unsigned gO2, unsigned gI1, unsigned gI2) const;
   std::complex<double> CpbarFdFdAhPR(unsigned gO1, unsigned gI1, unsigned gI2) const;
   std::complex<double> CpbarFdhhFdPL(unsigned gO2, unsigned gI1, unsigned gI2) const;
   std::complex<double> CpbarFdhhFdPR(unsigned gO1, unsigned gI1, unsigned gI2) const;
   std::complex<double> CpbarFdHpmFuPL(unsigned gO2, unsigned gI1, unsigned gI2) const;
   std::complex<double> CpbarFdHpmFuPR(unsigned gO1, unsigned gI1, unsigned gI2) const;
   std::complex<double> CpbarFdbarChiSdPL(unsigned gO2, unsigned gI1, unsigned gI2) const;
   std::complex<double> CpbarFdbarChiSdPR(unsigned gO1, unsigned gI1, unsigned gI2) const;
   double CpbarFdSuCha2PL(unsigned , unsigned , unsigned ) const;
   std::complex<double> CpbarFdSuCha2PR(unsigned gO1, unsigned gI1, unsigned gI2) const;
   std::complex<double> CpbarFdSdChiPL(unsigned gO2, unsigned gI1, unsigned gI2) const;
   std::complex<double> CpbarFdSdChiPR(unsigned gO1, unsigned gI1, unsigned gI2) const;
   std::complex<double> CpbarFdSdGluPL(unsigned gO2, unsigned gI1, unsigned ) const;
   double CpbarFdSdGluPR(unsigned , unsigned , unsigned ) const;
   double CpbarFdVWmFuPR(unsigned , unsigned ) const;
   std::complex<double> CpbarFdVWmFuPL(unsigned gO1, unsigned gI2) const;
   double CpbarFdVZFdPR(unsigned gO2, unsigned gI2) const;
   double CpbarFdVZFdPL(unsigned gO1, unsigned gI2) const;
   double CpbarFdbarGluSdPL(unsigned , unsigned , unsigned ) const;
   std::complex<double> CpbarFdbarGluSdPR(unsigned gO1, unsigned , unsigned gI2) const;
   std::complex<double> CpbarFubarCha2SdPL(unsigned gO2, unsigned gI1, unsigned gI2) const;
   std::complex<double> CpbarFubarCha2SdPR(unsigned gO1, unsigned gI1, unsigned gI2) const;
   std::complex<double> CpbarFuFuAhPL(unsigned gO2, unsigned gI1, unsigned gI2) const;
   std::complex<double> CpbarFuFuAhPR(unsigned gO1, unsigned gI1, unsigned gI2) const;
   std::complex<double> CpbarFuconjHpmFdPL(unsigned gO2, unsigned gI1, unsigned gI2) const;
   std::complex<double> CpbarFuconjHpmFdPR(unsigned gO1, unsigned gI1, unsigned gI2) const;
   std::complex<double> CpbarFuhhFuPL(unsigned gO2, unsigned gI1, unsigned gI2) const;
   std::complex<double> CpbarFuhhFuPR(unsigned gO1, unsigned gI1, unsigned gI2) const;
   std::complex<double> CpbarFubarChiSuPL(unsigned gO2, unsigned gI1, unsigned gI2) const;
   std::complex<double> CpbarFubarChiSuPR(unsigned gO1, unsigned gI1, unsigned gI2) const;
   double CpbarFuSdCha1PL(unsigned , unsigned , unsigned ) const;
   std::complex<double> CpbarFuSdCha1PR(unsigned gO1, unsigned gI1, unsigned gI2) const;
   std::complex<double> CpbarFuSuChiPL(unsigned gO2, unsigned gI1, unsigned gI2) const;
   std::complex<double> CpbarFuSuChiPR(unsigned gO1, unsigned gI1, unsigned gI2) const;
   std::complex<double> CpbarFuSuGluPL(unsigned gO2, unsigned gI1, unsigned ) const;
   double CpbarFuSuGluPR(unsigned , unsigned , unsigned ) const;
   double CpbarFuVPFuPR(unsigned gO2, unsigned gI2) const;
   double CpbarFuVPFuPL(unsigned gO1, unsigned gI2) const;
   double CpbarFuVZFuPR(unsigned gO2, unsigned gI2) const;
   double CpbarFuVZFuPL(unsigned gO1, unsigned gI2) const;
   double CpbarFuconjVWmFdPR(unsigned , unsigned ) const;
   std::complex<double> CpbarFuconjVWmFdPL(unsigned gO1, unsigned gI2) const;
   double CpbarFubarGluSuPL(unsigned , unsigned , unsigned ) const;
   std::complex<double> CpbarFubarGluSuPR(unsigned gO1, unsigned , unsigned gI2) const;
   std::complex<double> self_energy_Sd(double p , unsigned gO1, unsigned gO2) const;
   std::complex<double> self_energy_Sv(double p , unsigned gO1, unsigned gO2) const;
   std::complex<double> self_energy_Su(double p , unsigned gO1, unsigned gO2) const;
   std::complex<double> self_energy_Se(double p , unsigned gO1, unsigned gO2) const;
   std::complex<double> self_energy_hh(double p , unsigned gO1, unsigned gO2) const;
   std::complex<double> self_energy_Ah(double p , unsigned gO1, unsigned gO2) const;
   std::complex<double> self_energy_Rh(double p , unsigned gO1, unsigned gO2) const;
   std::complex<double> self_energy_Hpm(double p , unsigned gO1, unsigned gO2) const;
   std::complex<double> self_energy_Rpm(double p , unsigned gO1, unsigned gO2) const;
   std::complex<double> self_energy_SOc(double p ) const;
   std::complex<double> self_energy_VZ(double p ) const;
   std::complex<double> self_energy_VWm(double p ) const;
   std::complex<double> self_energy_Chi_1(double p , unsigned gO1, unsigned gO2) const;
   std::complex<double> self_energy_Chi_PR(double p , unsigned gO1, unsigned gO2) const;
   std::complex<double> self_energy_Chi_PL(double p , unsigned gO1, unsigned gO2) const;
   std::complex<double> self_energy_Cha1_1(double p , unsigned gO1, unsigned gO2) const;
   std::complex<double> self_energy_Cha1_PR(double p , unsigned gO1, unsigned gO2) const;
   std::complex<double> self_energy_Cha1_PL(double p , unsigned gO1, unsigned gO2) const;
   std::complex<double> self_energy_Cha2_1(double p , unsigned gO1, unsigned gO2) const;
   std::complex<double> self_energy_Cha2_PR(double p , unsigned gO1, unsigned gO2) const;
   std::complex<double> self_energy_Cha2_PL(double p , unsigned gO1, unsigned gO2) const;
   std::complex<double> self_energy_Fe_1(double p , unsigned gO1, unsigned gO2) const;
   std::complex<double> self_energy_Fe_PR(double p , unsigned gO1, unsigned gO2) const;
   std::complex<double> self_energy_Fe_PL(double p , unsigned gO1, unsigned gO2) const;
   std::complex<double> self_energy_Fd_1(double p , unsigned gO1, unsigned gO2) const;
   std::complex<double> self_energy_Fd_PR(double p , unsigned gO1, unsigned gO2) const;
   std::complex<double> self_energy_Fd_PL(double p , unsigned gO1, unsigned gO2) const;
   std::complex<double> self_energy_Fu_1(double p , unsigned gO1, unsigned gO2) const;
   std::complex<double> self_energy_Fu_PR(double p , unsigned gO1, unsigned gO2) const;
   std::complex<double> self_energy_Fu_PL(double p , unsigned gO1, unsigned gO2) const;
   std::complex<double> self_energy_Glu_1(double p ) const;
   std::complex<double> self_energy_Glu_PR(double p ) const;
   std::complex<double> self_energy_Glu_PL(double p ) const;
   std::complex<double> self_energy_VZ_heavy(double p ) const;
   std::complex<double> self_energy_VWm_heavy(double p ) const;
   std::complex<double> self_energy_Fe_1_heavy_rotated(double p , unsigned gO1, unsigned gO2) const;
   std::complex<double> self_energy_Fe_PR_heavy_rotated(double p , unsigned gO1, unsigned gO2) const;
   std::complex<double> self_energy_Fe_PL_heavy_rotated(double p , unsigned gO1, unsigned gO2) const;
   std::complex<double> self_energy_Fd_1_heavy_rotated(double p , unsigned gO1, unsigned gO2) const;
   std::complex<double> self_energy_Fd_PR_heavy_rotated(double p , unsigned gO1, unsigned gO2) const;
   std::complex<double> self_energy_Fd_PL_heavy_rotated(double p , unsigned gO1, unsigned gO2) const;
   std::complex<double> self_energy_Fu_1_heavy_rotated(double p , unsigned gO1, unsigned gO2) const;
   std::complex<double> self_energy_Fu_PR_heavy_rotated(double p , unsigned gO1, unsigned gO2) const;
   std::complex<double> self_energy_Fu_PL_heavy_rotated(double p , unsigned gO1, unsigned gO2) const;
   std::complex<double> tadpole_hh(unsigned gO1) const;




   void calculate_MGlu_pole();
   void calculate_MFv_pole();
   void calculate_MSOc_pole();
   void calculate_MVZ_pole();
   void calculate_MSd_pole();
   void calculate_MSv_pole();
   void calculate_MSu_pole();
   void calculate_MSe_pole();
   void calculate_Mhh_pole();
   void calculate_MAh_pole();
   void calculate_MRh_pole();
   void calculate_MHpm_pole();
   void calculate_MRpm_pole();
   void calculate_MChi_pole();
   void calculate_MCha1_pole();
   void calculate_MCha2_pole();
   void calculate_MFe_pole();
   void calculate_MFd_pole();
   void calculate_MFu_pole();
   void calculate_MVG_pole();
   void calculate_MVP_pole();
   void calculate_MVWm_pole();

   double calculate_MFu_DRbar(double, int) const;
   double calculate_MFd_DRbar(double, int) const;
   double calculate_MFe_DRbar(double, int) const;
   double calculate_MFv_DRbar(double, int) const;
   double calculate_MVP_DRbar(double);
   double calculate_MVZ_DRbar(double);
   double calculate_MVWm_DRbar(double);

   double ThetaW() const;
   double v() const;


private:
   struct Ewsb_parameters {
      MRSSM<Two_scale>* model;
      unsigned ewsb_loop_order;
   };

#ifdef ENABLE_THREADS
   struct Thread {
      typedef void(MRSSM<Two_scale>::*Memfun_t)();
      MRSSM<Two_scale>* model;
      Memfun_t fun;

      Thread(MRSSM<Two_scale>* model_, Memfun_t fun_)
         : model(model_), fun(fun_) {}
      void operator()() {
         try {
            (model->*fun)();
         } catch (...) {
            model->thread_exception = std::current_exception();
         }
      }
   };
#endif

   std::size_t number_of_ewsb_iterations;
   std::size_t number_of_mass_iterations;
   unsigned ewsb_loop_order;
   unsigned pole_mass_loop_order;
   bool calculate_sm_pole_masses; ///< switch to calculate the pole masses of the Standard Model particles
   double precision;              ///< RG running precision
   double ewsb_iteration_precision;
   static const std::size_t number_of_ewsb_equations = 4;
   MRSSM_physical physical; ///< contains the pole masses and mixings
   Problems<MRSSM_info::NUMBER_OF_PARTICLES> problems;
   Higgs_2loop_corrections higgs_2loop_corrections; ///< used Higgs 2-loop corrections
#ifdef ENABLE_THREADS
   std::exception_ptr thread_exception;
   static std::mutex mtx_fortran; /// locks fortran functions
#endif

   int solve_ewsb_iteratively();
   int solve_ewsb_iteratively(unsigned);
   int solve_ewsb_iteratively_with(const gsl_multiroot_fsolver_type*, const double[number_of_ewsb_equations]);
   int solve_ewsb_tree_level_via_soft_higgs_masses();
   void ewsb_initial_guess(double[number_of_ewsb_equations]);
   static int tadpole_equations(const gsl_vector*, void*, gsl_vector*);
   void copy_DRbar_masses_to_pole_masses();

   // Passarino-Veltman loop functions
   double A0(double) const;
   double B0(double, double, double) const;
   double B1(double, double, double) const;
   double B00(double, double, double) const;
   double B22(double, double, double) const;
   double H0(double, double, double) const;
   double F0(double, double, double) const;
   double G0(double, double, double) const;

   // DR-bar masses
   double MGlu;
   Eigen::Array<double,3,1> MFv;
   double MSOc;
   double MVZ;
   Eigen::Array<double,6,1> MSd;
   Eigen::Array<double,3,1> MSv;
   Eigen::Array<double,6,1> MSu;
   Eigen::Array<double,6,1> MSe;
   Eigen::Array<double,4,1> Mhh;
   Eigen::Array<double,4,1> MAh;
   Eigen::Array<double,2,1> MRh;
   Eigen::Array<double,4,1> MHpm;
   Eigen::Array<double,2,1> MRpm;
   Eigen::Array<double,4,1> MChi;
   Eigen::Array<double,2,1> MCha1;
   Eigen::Array<double,2,1> MCha2;
   Eigen::Array<double,3,1> MFe;
   Eigen::Array<double,3,1> MFd;
   Eigen::Array<double,3,1> MFu;
   double MVG;
   double MVP;
   double MVWm;

   // DR-bar mixing matrices
   Eigen::Matrix<double,6,6> ZD;
   Eigen::Matrix<double,3,3> ZV;
   Eigen::Matrix<double,6,6> ZU;
   Eigen::Matrix<double,6,6> ZE;
   Eigen::Matrix<double,4,4> ZH;
   Eigen::Matrix<double,4,4> ZA;
   Eigen::Matrix<double,2,2> ZHR;
   Eigen::Matrix<double,4,4> ZP;
   Eigen::Matrix<double,2,2> ZRP;
   Eigen::Matrix<std::complex<double>,4,4> ZN1;
   Eigen::Matrix<std::complex<double>,4,4> ZN2;
   Eigen::Matrix<std::complex<double>,2,2> UM1;
   Eigen::Matrix<std::complex<double>,2,2> UP1;
   Eigen::Matrix<std::complex<double>,2,2> UM2;
   Eigen::Matrix<std::complex<double>,2,2> UP2;
   Eigen::Matrix<std::complex<double>,3,3> ZEL;
   Eigen::Matrix<std::complex<double>,3,3> ZER;
   Eigen::Matrix<std::complex<double>,3,3> ZDL;
   Eigen::Matrix<std::complex<double>,3,3> ZDR;
   Eigen::Matrix<std::complex<double>,3,3> ZUL;
   Eigen::Matrix<std::complex<double>,3,3> ZUR;

   // phases

};

std::ostream& operator<<(std::ostream&, const MRSSM<Two_scale>&);

} // namespace flexiblesusy

#endif
